-- SET AUTOTRACE ON
SET TRIMSPOOL ON
SET TRIMOUT   ON
SET WRAP OFF

-- servicios usados por cada ciudadano
SELECT DNI_titular, tipoServ
FROM Tarjeta T, Accesos A, Servicio S
WHERE T.idTarjeta = A.idTarjeta AND A.idServ = S.idServ;

-- crear una tabla con los tipos de servicio usados por cada ciudadano
CREATE TABLE UsoServCiud AS (
  SELECT DISTINCT DNI_titular, tipoServ
  FROM  Tarjeta T, Accesos A, Servicio S
  WHERE T.idTarjeta = A.idTarjeta AND A.idServ = S.idServ
);
SELECT * FROM UsoServCiud;

/*                                                                                      (C1.1)
  ############################################################################################
  Parejas (sin repetir) de ciudadanos que han usado exactamente los mismos tipos de servicio.
  ##########################################################################################*/

SELECT DISTINCT C1.DNI_titular DNI_1, C2.DNI_titular DNI_2
FROM  UsoServCiud C1, UsoServCiud C2
WHERE C1.DNI_titular < C2.DNI_titular AND
  NOT EXISTS((SELECT tipoServ FROM UsoServCiud WHERE DNI_titular = C1.DNI_titular
              MINUS
              SELECT tipoServ FROM UsoServCiud WHERE DNI_titular = C2.DNI_titular)
              UNION
             (SELECT tipoServ FROM UsoServCiud WHERE DNI_titular = C2.DNI_titular
              MINUS
              SELECT tipoServ FROM UsoServCiud WHERE DNI_titular = C1.DNI_titular));

/*                                                                                      (C1.2)
  ##########################################################################################*/

SELECT DISTINCT C1.DNI_titular, C2.DNI_titular
FROM  UsoServCiud C1, UsoServCiud C2
WHERE C1.DNI_titular < C2.DNI_titular AND
  NOT EXISTS((SELECT tipoServ FROM UsoServCiud
              WHERE DNI_titular = C1.DNI_titular OR DNI_titular = C2.DNI_titular
              MINUS
              SELECT S1.tipoServ FROM UsoServCiud S1, UsoServCiud S2
              WHERE S1.DNI_titular = C1.DNI_titular AND S2.DNI_titular = C2.DNI_titular
                AND S1.tipoServ = S2.tipoServ));

/*                                                                                      (C1.3)
  ##########################################################################################*/

SELECT DISTINCT C1.DNI_titular, C2.DNI_titular
FROM  UsoServCiud C1, UsoServCiud C2
WHERE C1.DNI_titular < C2.DNI_titular AND
  NOT EXISTS (SELECT tipoServ FROM UsoServCiud
              WHERE DNI_titular = C1.DNI_titular OR DNI_titular = C2.DNI_titular
              MINUS(
              SELECT tipoServ FROM UsoServCiud WHERE DNI_titular = C1.DNI_titular
              INTERSECT
              SELECT tipoServ FROM UsoServCiud WHERE DNI_titular = C2.DNI_titular));

/*                                                                                      (C1.4)
  ##########################################################################################*/

SELECT DISTINCT C1.DNI_titular, C2.DNI_titular FROM  UsoServCiud C1, UsoServCiud C2
WHERE C1.DNI_titular < C2.DNI_titular
MINUS
SELECT C1.DNI_titular, C2.DNI_titular FROM UsoServCiud C1, UsoServCiud C2
WHERE EXISTS (SELECT tipoServ FROM UsoServCiud
              WHERE DNI_titular = C1.DNI_titular
                AND tipoServ NOT IN (SELECT tipoServ FROM UsoServCiud
                                     WHERE DNI_titular =C2.DNI_titular))
   OR EXISTS (SELECT tipoServ FROM UsoServCiud
              WHERE DNI_titular = C2.DNI_titular
                AND tipoServ NOT IN (SELECT tipoServ FROM UsoServCiud
                                     WHERE DNI_titular =C1.DNI_titular));

/*                                                                                      (C1.5)
  ##########################################################################################*/

SELECT DISTINCT C1.DNI_titular, C2.DNI_titular
FROM  UsoServCiud C1, UsoServCiud C2  WHERE C1.DNI_titular < C2.DNI_titular
MINUS
SELECT S1.DNI_titular, S2.DNI_titular FROM UsoServCiud S1, UsoServCiud S2
WHERE S1.tipoServ NOT IN (SELECT tipoServ FROM UsoServCiud WHERE DNI_titular = S2.DNI_titular)
   OR S2.tipoServ NOT IN (SELECT tipoServ FROM UsoServCiud WHERE DNI_titular = S1.DNI_titular);

/*                                                                                      (C1.6)
  ##########################################################################################*/
/* crear una vista con las parejas que comparten el uso de algn tipo de servicio, junto con
   el nmero de tipos de servicio que comparten */

CREATE VIEW nServAmbos (DNI_titular1, DNI_titular2, N) AS
 SELECT S1.DNI_titular, S2.DNI_titular, count(*) FROM  UsoServCiud S1, UsoServCiud S2
 WHERE S1.DNI_titular < S2.DNI_titular AND S1.tipoServ = S2.tipoServ
 GROUP BY S1.DNI_titular, S2.DNI_titular;

SELECT * FROM nServAmbos;

/* crear una vista con el nmero de tipos de servicio que ha usado cada ciudadano */
CREATE VIEW nServCiud (DNI_titular, N) AS
 SELECT DNI_titular, count(*) FROM  UsoServCiud
 GROUP BY DNI_titular;

SELECT * FROM nServCiud;

SELECT DNI_titular1, DNI_titular2 FROM  nServAmbos, nServCiud nSrv1, nServCiud nSrv2
WHERE DNI_titular1=nSrv1.DNI_titular AND DNI_titular2=nSrv2.DNI_titular
  AND nServAmbos.N=nSrv1.N AND nServAmbos.N=nSrv2.N;

DROP VIEW nServAmbos;
DROP VIEW nServCiud;

/*                                                                                      (C1.7)
  ##########################################################################################*/
/* sin utilizar las vistas anteriores */
SELECT S1.DNI_titular, S2.DNI_titular FROM  UsoServCiud S1, UsoServCiud S2
WHERE S1.DNI_titular < S2.DNI_titular AND S1.tipoServ = S2.tipoServ
GROUP BY S1.DNI_titular, S2.DNI_titular
HAVING count(*) = (SELECT count(*) FROM UsoServCiud WHERE DNI_titular = S1.DNI_titular)
   AND count(*) = (SELECT count(*) FROM UsoServCiud WHERE DNI_titular = S2.DNI_titular);

/*                                                                                      (c1.8)
  ##########################################################################################*/

SELECT C1.DNI_titular DNI_1, C2.DNI_titular DNI_2
FROM  UsoServCiud c1, UsoServCiud c2
WHERE C1.DNI_titular < C2.DNI_titular
MINUS
SELECT DNI_1, DNI_2
FROM (SELECT C1.DNI_titular DNI_1, C2.DNI_titular DNI_2, C1.tipoServ
      FROM  UsoServCiud c1, UsoServCiud c2
      MINUS
      SELECT C1.DNI_titular DNI_1, C2.DNI_titular DNI_2, C2.tipoServ
      FROM  UsoServCiud c1, UsoServCiud c2)
MINUS
SELECT DNI_1, DNI_2
FROM (SELECT C1.DNI_titular DNI_1, C2.DNI_titular DNI_2, C2.tipoServ
      FROM  UsoServCiud c1, UsoServCiud c2
      MINUS
      SELECT C1.DNI_titular DNI_1, C2.DNI_titular DNI_2, C1.tipoServ
      FROM  UsoServCiud c1, UsoServCiud c2);


/*                                                                                      (c1.9)
  ##########################################################################################*/

SELECT C1.DNI_titular DNI_1, C2.DNI_titular DNI_2
FROM  UsoServCiud c1, UsoServCiud c2
WHERE C1.DNI_titular < C2.DNI_titular
MINUS
SELECT DNI_1, DNI_2
FROM (SELECT C1.DNI_titular DNI_1, C2.DNI_titular DNI_2, C1.tipoServ
      FROM  UsoServCiud c1, UsoServCiud c2 WHERE C1.DNI_titular < C2.DNI_titular
      MINUS
      SELECT C1.DNI_titular DNI_1, C2.DNI_titular DNI_2, C2.tipoServ
      FROM  UsoServCiud c1, UsoServCiud c2 WHERE C1.DNI_titular < C2.DNI_titular)
MINUS
SELECT DNI_1, DNI_2
FROM (SELECT C1.DNI_titular DNI_1, C2.DNI_titular DNI_2, C2.tipoServ
      FROM  UsoServCiud c1, UsoServCiud c2 WHERE C1.DNI_titular < C2.DNI_titular
      MINUS
      SELECT C1.DNI_titular DNI_1, C2.DNI_titular DNI_2, C1.tipoServ
      FROM  UsoServCiud c1, UsoServCiud c2 WHERE C1.DNI_titular < C2.DNI_titular);

/*==========================================================================================*/
/* deshacer los cambios realizados */

-- eliminar la tabla creada
DROP TABLE UsoServCiud;
